>From dbc4a8091e4794fca911330a8c7bfe594e22aa82 Mon Sep 17 00:00:00 2001
From: Joseph Tulou <brezhoneg1@yahoo.fr>
Date: Fri, 30 Jan 2009 01:06:36 +0100
Subject: [PATCH] Fullscreen enhancements
    - resizing mechanism in fullscreen mode (scaling and zoom)
    - transition problems between items in fullscreen mode worked out
---
 include/vlc_vout.h                   |    7 ++
 modules/control/hotkeys.c            |   12 ++-
 modules/video_filter/filter_common.h |    4 +
 modules/video_output/msw/direct3d.c  |   23 ++++
 modules/video_output/msw/directx.c   |   23 ++++
 modules/video_output/msw/glwin32.c   |   23 ++++
 modules/video_output/msw/wingdi.c    |   23 ++++
 modules/video_output/opengl.c        |   16 +++-
 modules/video_output/sdl.c           |   23 ++++
 modules/video_output/x11/xcommon.c   |   45 ++++++++
 src/video_output/video_output.c      |  194 +++++++++++++++++-----------------
 src/video_output/vout_intf.c         |   44 +++++---
 src/video_output/vout_pictures.c     |   14 ++-
 13 files changed, 332 insertions(+), 119 deletions(-)

diff --git a/include/vlc_vout.h b/include/vlc_vout.h
index adab0a6..0387dd5 100644
--- a/include/vlc_vout.h
+++ b/include/vlc_vout.h
@@ -507,6 +507,7 @@ struct vout_thread_t
     uint16_t            i_changes;          /**< changes made to the thread.
                                                       \see \ref vout_changes */
     bool                b_scale;                  /**< allow picture scaling */
+    uint64_t            ll_zoom;                  /**< allow picture Zooming */
     bool                b_fullscreen;         /**< toogle fullscreen display */
     unsigned int        i_window_width;              /**< video window width */
     unsigned int        i_window_height;            /**< video window height */
@@ -572,6 +573,8 @@ struct vout_thread_t
 #define VOUT_CURSOR_CHANGE      0x0020
 /** b_fullscreen changed */
 #define VOUT_FULLSCREEN_CHANGE  0x0040
+/** ll_zoom changed */
+#define VOUT_ZOOM_CHANGE        0x0080
 /** size changed */
 #define VOUT_SIZE_CHANGE        0x0200
 /** depth changed */
@@ -596,6 +599,9 @@ struct vout_thread_t
 
 #define MAX_JITTER_SAMPLES      20
 
+/* zoom factor (applied to ll_zoom in vout_thread_t) */
+#define ZOOM_FP_FACTOR          1000
+
 /*****************************************************************************
  * Prototypes
  *****************************************************************************/
@@ -670,6 +676,7 @@ VLC_EXPORT( void,            vout_PlacePicture,   ( const vout_thread_t *, unsig
 
 VLC_EXPORT( int, vout_vaControlDefault, ( vout_thread_t *, int, va_list ) );
 void vout_IntfInit( vout_thread_t * );
+void vout_InitWindowSize( vout_thread_t *, unsigned *, unsigned * );
 VLC_EXPORT( int, vout_Snapshot, ( vout_thread_t *p_vout, picture_t *p_pic ) );
 VLC_EXPORT( void, vout_EnableFilter, ( vout_thread_t *, char *,bool , bool  ) );
 
diff --git a/modules/control/hotkeys.c b/modules/control/hotkeys.c
index b11e8a6..93803b7 100644
--- a/modules/control/hotkeys.c
+++ b/modules/control/hotkeys.c
@@ -611,8 +611,14 @@ static void Run( intf_thread_t *p_intf )
             }
             else if( i_action == ACTIONID_TOGGLE_SCALING && p_vout )
             {
-                bool b_scaling = var_GetBool( p_vout, "scaling" );
-                var_SetBool( p_vout, "scaling", !b_scaling );
+                bool b_scaling = !var_GetBool( p_vout, "scaling" );
+                var_SetBool( p_vout, "scaling", b_scaling );
+                if( b_scaling )
+                    vout_OSDMessage( VLC_OBJECT(p_input), DEFAULT_CHAN,
+                                     _("Scaled to screen") );
+                else
+                    vout_OSDMessage( VLC_OBJECT(p_input), DEFAULT_CHAN,
+                                     _("Original Size") );
             }
             else if( i_action == ACTIONID_DEINTERLACE && p_vout )
             {
@@ -667,7 +673,7 @@ static void Run( intf_thread_t *p_intf )
                     var_SetFloat( p_vout, "zoom",
                                   val_list.p_list->p_values[i].f_float );
                     vout_OSDMessage( VLC_OBJECT(p_input), DEFAULT_CHAN,
-                                     _("Zoom mode: %s"),
+                                     _("Zoom: %s"),
                                 text_list.p_list->p_values[i].var.psz_name );
 
                     var_Change( p_vout, "zoom", VLC_VAR_FREELIST, &val_list, &text_list );
diff --git a/modules/video_filter/filter_common.h b/modules/video_filter/filter_common.h
index 2efc052..dc98b44 100644
--- a/modules/video_filter/filter_common.h
+++ b/modules/video_filter/filter_common.h
@@ -90,11 +90,15 @@ static int SetParentVal( vlc_object_t *p_this, char const *psz_var,
 
 #define ADD_PARENT_CALLBACKS( handler ) \
     var_AddCallback( p_vout, "fullscreen", handler, NULL );                   \
+    var_AddCallback( p_vout, "scaling", handler, NULL );                      \
+    var_AddCallback( p_vout, "zoom", handler, NULL );                      \
     var_AddCallback( p_vout, "aspect-ratio", handler, NULL );                 \
     var_AddCallback( p_vout, "crop", handler, NULL );
 
 #define DEL_PARENT_CALLBACKS( handler ) \
     var_DelCallback( p_vout, "fullscreen", handler, NULL );                   \
+    var_DelCallback( p_vout, "scaling", handler, NULL );                      \
+    var_DelCallback( p_vout, "zoom", handler, NULL );                      \
     var_DelCallback( p_vout, "aspect-ratio", handler, NULL );                 \
     var_DelCallback( p_vout, "crop", handler, NULL );
 
diff --git a/modules/video_output/msw/direct3d.c b/modules/video_output/msw/direct3d.c
index 5797927..c56258d 100644
--- a/modules/video_output/msw/direct3d.c
+++ b/modules/video_output/msw/direct3d.c
@@ -456,6 +456,29 @@ static int Manage( vout_thread_t *p_vout )
         p_vout->p_sys->i_changes &= ~DX_POSITION_CHANGE;
     }
 
+    /* scaling/no-scaling toggle */
+    if( p_vout->i_changes & VOUT_SCALE_CHANGE )
+    {
+        p_vout->i_changes &= ~VOUT_SCALE_CHANGE;
+
+        p_vout->b_scale = var_GetBool( p_vout, "scaling" );
+        p_vout->ll_zoom = (uint64_t) ZOOM_FP_FACTOR;
+
+        UpdateRects( p_vout, true );
+    }
+
+    /* zoom factor (only occurs in fullscreen mode) */
+    if( p_vout->i_changes & VOUT_ZOOM_CHANGE )
+    {
+        p_vout->i_changes &= ~VOUT_ZOOM_CHANGE;
+
+        p_vout->b_scale = false;
+        p_vout->ll_zoom =
+            (uint64_t)( ZOOM_FP_FACTOR * var_GetFloat( p_vout, "zoom" ) );
+
+        UpdateRects( p_vout, true );
+    }
+
     /* Check for cropping / aspect changes */
     if( p_vout->i_changes & VOUT_CROP_CHANGE ||
         p_vout->i_changes & VOUT_ASPECT_CHANGE )
diff --git a/modules/video_output/msw/directx.c b/modules/video_output/msw/directx.c
index a588ad4..8f803bf 100644
--- a/modules/video_output/msw/directx.c
+++ b/modules/video_output/msw/directx.c
@@ -602,6 +602,29 @@ static int Manage( vout_thread_t *p_vout )
         }
     }
 
+    /* scaling/no-scaling toggle */
+    if( p_vout->i_changes & VOUT_SCALE_CHANGE )
+    {
+        p_vout->i_changes &= ~VOUT_SCALE_CHANGE;
+
+        p_vout->b_scale = var_GetBool( p_vout, "scaling" );
+        p_vout->ll_zoom = (uint64_t) ZOOM_FP_FACTOR;
+
+        UpdateRects( p_vout, true );
+    }
+
+    /* zoom factor (only occurs in fullscreen mode) */
+    if( p_vout->i_changes & VOUT_ZOOM_CHANGE )
+    {
+        p_vout->i_changes &= ~VOUT_ZOOM_CHANGE;
+
+        p_vout->b_scale = false;
+        p_vout->ll_zoom =
+            (uint64_t)( ZOOM_FP_FACTOR * var_GetFloat( p_vout, "zoom" ) );
+
+        UpdateRects( p_vout, true );
+    }
+
     /* Check for cropping / aspect changes */
     if( p_vout->i_changes & VOUT_CROP_CHANGE ||
         p_vout->i_changes & VOUT_ASPECT_CHANGE )
diff --git a/modules/video_output/msw/glwin32.c b/modules/video_output/msw/glwin32.c
index 10f6bd8..bb437e1 100644
--- a/modules/video_output/msw/glwin32.c
+++ b/modules/video_output/msw/glwin32.c
@@ -305,6 +305,29 @@ static int Manage( vout_thread_t *p_vout )
         vlc_mutex_unlock( &p_vout->p_sys->lock );
     }
 
+    /* scaling/no-scaling toggle */
+    if( p_vout->i_changes & VOUT_SCALE_CHANGE )
+    {
+        p_vout->i_changes &= ~VOUT_SCALE_CHANGE;
+
+        p_vout->b_scale = var_GetBool( p_vout, "scaling" );
+        p_vout->ll_zoom = (uint64_t) ZOOM_FP_FACTOR;
+
+        UpdateRects( p_vout, true );
+    }
+
+    /* zoom factor (only occurs in fullscreen mode) */
+    if( p_vout->i_changes & VOUT_ZOOM_CHANGE )
+    {
+        p_vout->i_changes &= ~VOUT_ZOOM_CHANGE;
+
+        p_vout->b_scale = false;
+        p_vout->ll_zoom =
+            (uint64_t)( ZOOM_FP_FACTOR * var_GetFloat( p_vout, "zoom" ) );
+
+        UpdateRects( p_vout, true );
+    }
+
     /* Check for cropping / aspect changes */
     if( p_vout->i_changes & VOUT_CROP_CHANGE ||
         p_vout->i_changes & VOUT_ASPECT_CHANGE )
diff --git a/modules/video_output/msw/wingdi.c b/modules/video_output/msw/wingdi.c
index ef41f3f..c7bd0d6 100644
--- a/modules/video_output/msw/wingdi.c
+++ b/modules/video_output/msw/wingdi.c
@@ -522,6 +522,29 @@ static int Manage( vout_thread_t *p_vout )
         vlc_mutex_unlock( &p_vout->p_sys->lock );
     }
 
+    /* scaling/no-scaling toggle */
+    if( p_vout->i_changes & VOUT_SCALE_CHANGE )
+    {
+        p_vout->i_changes &= ~VOUT_SCALE_CHANGE;
+
+        p_vout->b_scale = var_GetBool( p_vout, "scaling" );
+        p_vout->ll_zoom = (uint64_t) ZOOM_FP_FACTOR;
+
+        UpdateRects( p_vout, true );
+    }
+
+    /* zoom factor (only occurs in fullscreen mode) */
+    if( p_vout->i_changes & VOUT_ZOOM_CHANGE )
+    {
+        p_vout->i_changes &= ~VOUT_ZOOM_CHANGE;
+
+        p_vout->b_scale = false;
+        p_vout->ll_zoom =
+            (uint64_t)( ZOOM_FP_FACTOR * var_GetFloat( p_vout, "zoom" ) );
+
+        UpdateRects( p_vout, true );
+    }
+
     /* Check for cropping / aspect changes */
     if( p_vout->i_changes & VOUT_CROP_CHANGE ||
         p_vout->i_changes & VOUT_ASPECT_CHANGE )
diff --git a/modules/video_output/opengl.c b/modules/video_output/opengl.c
index 1e8121b..bc27ca3 100644
--- a/modules/video_output/opengl.c
+++ b/modules/video_output/opengl.c
@@ -224,12 +224,15 @@ static int CreateVout( vlc_object_t *p_this )
     var_Create( p_sys->p_vout, "mouse-button-down", VLC_VAR_INTEGER );
     var_Create( p_sys->p_vout, "video-on-top",
                 VLC_VAR_BOOL | VLC_VAR_DOINHERIT );
+    var_Create( p_sys->p_vout, "scaling",
+                VLC_VAR_BOOL | VLC_VAR_DOINHERIT );
 
     var_AddCallback( p_sys->p_vout, "mouse-x", SendEvents, p_vout );
     var_AddCallback( p_sys->p_vout, "mouse-y", SendEvents, p_vout );
     var_AddCallback( p_sys->p_vout, "mouse-moved", SendEvents, p_vout );
     var_AddCallback( p_sys->p_vout, "mouse-clicked", SendEvents, p_vout );
     var_AddCallback( p_sys->p_vout, "mouse-button-down", SendEvents, p_vout );
+    var_AddCallback( p_vout, "scaling", SendEvents, p_sys->p_vout );
 
     return VLC_SUCCESS;
 }
@@ -454,9 +457,18 @@ static int Manage( vout_thread_t *p_vout )
 // to align in real time in OPENGL
     if (p_sys->p_vout->i_alignment != p_vout->i_alignment)
     {
-        p_vout->i_changes = VOUT_CROP_CHANGE;        //to force change
+        p_vout->i_changes |= VOUT_CROP_CHANGE;        //to force change
         p_sys->p_vout->i_alignment = p_vout->i_alignment;    
     }
+
+    /* forward signal that scaling toggle has changed */
+    if (p_vout->i_changes & VOUT_SCALE_CHANGE )
+    {
+        p_vout->i_changes &= ~VOUT_SCALE_CHANGE;
+ 
+        p_sys->p_vout->i_changes |= VOUT_SCALE_CHANGE;
+    }
+
     return i_ret;
 }
 
@@ -664,7 +676,7 @@ static int InitTextures( vout_thread_t *p_vout )
 }
 
 /*****************************************************************************
- * SendEvents: forward mouse and keyboard events to the parent p_vout
+ * SendEvents: forward mouse, keyboard and scaling events to the parent p_vout
  *****************************************************************************/
 static int SendEvents( vlc_object_t *p_this, char const *psz_var,
                        vlc_value_t oldval, vlc_value_t newval, void *_p_vout )
diff --git a/modules/video_output/sdl.c b/modules/video_output/sdl.c
index 187e539..2b190ba 100644
--- a/modules/video_output/sdl.c
+++ b/modules/video_output/sdl.c
@@ -596,6 +596,29 @@ static int Manage( vout_thread_t *p_vout )
         p_vout->i_changes |= VOUT_SIZE_CHANGE;
     }
 
+    /* scaling/no-scaling toggle */
+    if( p_vout->i_changes & VOUT_SCALE_CHANGE )
+    {
+        p_vout->i_changes &= ~VOUT_SCALE_CHANGE;
+
+        p_vout->b_scale = var_GetBool( p_vout, "scaling" );
+        p_vout->ll_zoom = (uint64_t) ZOOM_FP_FACTOR;
+
+        p_vout->i_changes |= VOUT_SIZE_CHANGE;
+    }
+
+    /* zoom factor (only occurs in fullscreen mode) */
+    if( p_vout->i_changes & VOUT_ZOOM_CHANGE )
+    {
+        p_vout->i_changes &= ~VOUT_ZOOM_CHANGE;
+
+        p_vout->b_scale = false;
+        p_vout->ll_zoom =
+            (uint64_t)( ZOOM_FP_FACTOR * var_GetFloat( p_vout, "zoom" ) );
+
+        p_vout->i_changes |= VOUT_SIZE_CHANGE;
+    }
+
     /* Size change */
     if( p_vout->i_changes & VOUT_SIZE_CHANGE )
     {
diff --git a/modules/video_output/x11/xcommon.c b/modules/video_output/x11/xcommon.c
index 5ba486d..0a6d1bf 100644
--- a/modules/video_output/x11/xcommon.c
+++ b/modules/video_output/x11/xcommon.c
@@ -1243,6 +1243,28 @@ static int ManageVideo( vout_thread_t *p_vout )
                     val.i_int |= KEY_MODIFIER_ALT;
                 }
                 var_Set( p_vout->p_libvlc, "key-pressed", val );
+
+                if( p_vout->b_fullscreen ) 
+                {
+                    if( val.i_int & KEY_MODIFIER_SHIFT &&
+                        val.i_int & KEY_MODIFIER_ALT &&
+                        val.i_int & 'o' )
+                    {
+                        if( p_vout->ll_zoom <= 4000 )
+                            p_vout->ll_zoom += 50;
+                        p_vout->b_scale = false;
+                        p_vout->i_changes |= VOUT_SIZE_CHANGE;
+                    }
+                    if( val.i_int & KEY_MODIFIER_CTRL  &&
+                        val.i_int & KEY_MODIFIER_ALT &&
+                        val.i_int & 'o' )
+                    {
+                        if( p_vout->ll_zoom > 100 )
+                            p_vout->ll_zoom -= 50;
+                        p_vout->b_scale = false;
+                        p_vout->i_changes |= VOUT_SIZE_CHANGE;
+                    }
+                }
             }
         }
         /* Mouse click */
@@ -1478,6 +1500,29 @@ static int ManageVideo( vout_thread_t *p_vout )
         p_vout->i_changes &= ~VOUT_FULLSCREEN_CHANGE;
     }
 
+    /* scaling/no-scaling toggle */
+    if( p_vout->i_changes & VOUT_SCALE_CHANGE )
+    {
+        p_vout->i_changes &= ~VOUT_SCALE_CHANGE;
+
+        p_vout->b_scale = var_GetBool( p_vout, "scaling" );
+        p_vout->ll_zoom = (uint64_t) ZOOM_FP_FACTOR;
+
+        p_vout->i_changes |= VOUT_SIZE_CHANGE;
+    }
+
+    /* zoom factor (only occurs in fullscreen mode) */
+    if( p_vout->i_changes & VOUT_ZOOM_CHANGE )
+    {
+        p_vout->i_changes &= ~VOUT_ZOOM_CHANGE;
+
+        p_vout->b_scale = false;
+        p_vout->ll_zoom =
+            (uint64_t)( ZOOM_FP_FACTOR * var_GetFloat( p_vout, "zoom" ) );
+
+        p_vout->i_changes |= VOUT_SIZE_CHANGE;
+    }
+
     if( p_vout->i_changes & VOUT_CROP_CHANGE ||
         p_vout->i_changes & VOUT_ASPECT_CHANGE )
     {
diff --git a/src/video_output/video_output.c b/src/video_output/video_output.c
index 27de388..ef53092 100644
--- a/src/video_output/video_output.c
+++ b/src/video_output/video_output.c
@@ -77,6 +77,7 @@ static void VideoFormatImportRgb( video_format_t *, const picture_heap_t * );
 static void PictureHeapFixRgb( picture_heap_t * );
 
 static void     vout_Destructor   ( vlc_object_t * p_this );
+static void vout_Reset( vout_thread_t* p_vout, video_format_t *p_fmt );
 
 /* Object variables callbacks */
 static int DeinterlaceCallback( vlc_object_t *, char const *,
@@ -155,127 +156,43 @@ vout_thread_t *__vout_Request( vlc_object_t *p_this, vout_thread_t *p_vout,
 {
     if( !p_fmt )
     {
-        /* Video output is no longer used.
-         * TODO: support for reusing video outputs with proper _thread-safe_
-         * reference handling. */
         if( p_vout )
             vout_CloseAndRelease( p_vout );
         return NULL;
     }
 
-    /* If a video output was provided, lock it, otherwise look for one. */
     if( p_vout )
     {
-        vlc_object_hold( p_vout );
-    }
-
-    /* TODO: find a suitable unused video output */
-
-    /* If we now have a video output, check it has the right properties */
-    if( p_vout )
-    {
-        char *psz_filter_chain;
-        vlc_value_t val;
-
         vlc_mutex_lock( &p_vout->change_lock );
 
-        /* We don't directly check for the "vout-filter" variable for obvious
-         * performance reasons. */
-        if( p_vout->p->b_filter_change )
-        {
-            var_Get( p_vout, "vout-filter", &val );
-            psz_filter_chain = val.psz_string;
-
-            if( psz_filter_chain && !*psz_filter_chain )
-            {
-                free( psz_filter_chain );
-                psz_filter_chain = NULL;
-            }
-            if( p_vout->p->psz_filter_chain && !*p_vout->p->psz_filter_chain )
-            {
-                free( p_vout->p->psz_filter_chain );
-                p_vout->p->psz_filter_chain = NULL;
-            }
-
-            if( !psz_filter_chain && !p_vout->p->psz_filter_chain )
-            {
-                p_vout->p->b_filter_change = false;
-            }
-
-            free( psz_filter_chain );
-        }
-
-        if( p_vout->fmt_render.i_chroma != p_fmt->i_chroma ||
-            p_vout->fmt_render.i_width != p_fmt->i_width ||
-            p_vout->fmt_render.i_height != p_fmt->i_height ||
-            p_vout->p->b_filter_change )
+        if( p_vout->p->psz_filter_chain && *p_vout->p->psz_filter_chain )
         {
+            free( p_vout->p->psz_filter_chain );
+            p_vout->p->psz_filter_chain = NULL;
+    
             vlc_mutex_unlock( &p_vout->change_lock );
 
-            /* We are not interested in this format, close this vout */
             vout_CloseAndRelease( p_vout );
-            vlc_object_release( p_vout );
             p_vout = NULL;
         }
         else
         {
-            /* This video output is cool! Hijack it. */
-            if( p_vout->fmt_render.i_aspect != p_fmt->i_aspect )
-            {
-                /* Correct aspect ratio on change
-                 * FIXME factorize this code with other aspect ration related code */
-                unsigned int i_sar_num;
-                unsigned int i_sar_den;
-                unsigned int i_aspect;
-
-                i_aspect = p_fmt->i_aspect;
-                vlc_ureduce( &i_sar_num, &i_sar_den,
-                             p_fmt->i_sar_num, p_fmt->i_sar_den, 50000 );
-#if 0
-                /* What's that, it does not seems to be used correcly everywhere
-                 * beside the previous p_vout->fmt_render.i_aspect != p_fmt->i_aspect
-                 * should be fixed to use it too then */
-                if( p_vout->i_par_num > 0 && p_vout->i_par_den > 0 )
-                {
-                    i_sar_num *= p_vout->i_par_den;
-                    i_sar_den *= p_vout->i_par_num;
-                    i_aspect = i_aspect * p_vout->i_par_den / p_vout->i_par_num;
-                }
-#endif
-
-                if( i_sar_num > 0 && i_sar_den > 0 && i_aspect > 0 )
-                {
-                    p_vout->fmt_in.i_sar_num = i_sar_num;
-                    p_vout->fmt_in.i_sar_den = i_sar_den;
-                    p_vout->fmt_in.i_aspect  = i_aspect;
-
-                    p_vout->fmt_render.i_sar_num = i_sar_num;
-                    p_vout->fmt_render.i_sar_den = i_sar_den;
-                    p_vout->fmt_render.i_aspect  = i_aspect;
-
-                    p_vout->render.i_aspect   = i_aspect;
-
-                    p_vout->i_changes |= VOUT_ASPECT_CHANGE;
-
-                }
-            }
             vlc_mutex_unlock( &p_vout->change_lock );
-
-            vlc_object_release( p_vout );
         }
+    }
 
-        if( p_vout )
-        {
-            msg_Dbg( p_this, "reusing provided vout" );
+    if( p_vout )
+    {
+        msg_Dbg( p_this, "reusing provided vout" );
 
-            spu_Attach( p_vout->p_spu, p_this, true );
+        vout_Reset( p_vout, p_fmt );
 
-            vlc_object_detach( p_vout );
-            vlc_object_attach( p_vout, p_this );
-        }
-    }
+        spu_Attach( p_vout->p_spu, p_this, true );
 
-    if( !p_vout )
+        vlc_object_detach( p_vout );
+        vlc_object_attach( p_vout, p_this );
+    }
+    else
     {
         msg_Dbg( p_this, "no usable vout present, spawning one" );
 
@@ -375,6 +292,7 @@ vout_thread_t * __vout_Create( vlc_object_t *p_parent, video_format_t *p_fmt )
     /* Initialize misc stuff */
     p_vout->i_changes    = 0;
     p_vout->b_scale      = 1;
+    p_vout->ll_zoom      = ZOOM_FP_FACTOR;
     p_vout->b_fullscreen = 0;
     p_vout->i_alignment  = 0;
     p_vout->p->render_time  = 10;
@@ -1733,3 +1651,83 @@ static void DisplayTitleOnOSD( vout_thread_t *p_vout )
     p_vout->p->psz_title = NULL;
 }
 
+/*
+ * Reset a previous vout and take new input format into account
+ */
+void vout_Reset( vout_thread_t* p_vout, video_format_t *p_fmt )
+{
+    vlc_mutex_lock( &p_vout->change_lock );
+    
+    /* clean up data from previous use of vout */
+    CleanThread( p_vout );
+
+    /* set data from new input */
+    unsigned int i_width = p_fmt->i_width;
+    unsigned int i_height = p_fmt->i_height;
+    vlc_fourcc_t i_chroma = p_fmt->i_chroma;
+    unsigned int i_aspect = p_fmt->i_aspect;
+
+    if( i_width <= 0 || i_height <= 0 || i_aspect <= 0 )
+        return;
+
+    vlc_ureduce( &p_fmt->i_sar_num, &p_fmt->i_sar_den,
+                 p_fmt->i_sar_num, p_fmt->i_sar_den, 50000 );
+    if( p_fmt->i_sar_num <= 0 || p_fmt->i_sar_den <= 0 )
+        return;
+
+    /* Initialize pictures - translation tables and functions
+     * will be initialized later in InitThread */
+    for( int i_index = 0; i_index < 2 * VOUT_MAX_PICTURES + 1; i_index++)
+    {
+        p_vout->p_picture[i_index].i_status = FREE_PICTURE;
+        p_vout->p_picture[i_index].i_type   = EMPTY_PICTURE;
+        p_vout->p_picture[i_index].b_slow   = 0;
+    }
+
+    /* No images in the heap */
+    p_vout->i_heap_size = 0;
+
+    /* Initialize the rendering heap */
+    I_RENDERPICTURES = 0;
+
+    p_vout->fmt_render        = *p_fmt;   /* FIXME palette */
+    p_vout->fmt_in            = *p_fmt;   /* FIXME palette */
+
+    p_vout->render.i_width    = i_width;
+    p_vout->render.i_height   = i_height;
+    p_vout->render.i_chroma   = i_chroma;
+    p_vout->render.i_aspect   = i_aspect;
+
+    p_vout->render.i_rmask    = p_fmt->i_rmask;
+    p_vout->render.i_gmask    = p_fmt->i_gmask;
+    p_vout->render.i_bmask    = p_fmt->i_bmask;
+
+    p_vout->render.i_last_used_pic = -1;
+    p_vout->render.b_allow_modify_pics = 1;
+
+    /* Zero the output heap */
+    I_OUTPUTPICTURES = 0;
+    p_vout->output.i_width    = 0;
+    p_vout->output.i_height   = 0;
+    // p_vout->output.i_chroma   = 0;
+    p_vout->output.i_aspect   = 0;
+
+    //p_vout->output.i_rmask    = 0;
+    //p_vout->output.i_gmask    = 0;
+    //p_vout->output.i_bmask    = 0;
+
+    /* Reset window size */
+    vout_InitWindowSize( p_vout, &p_vout->i_window_width,
+                         &p_vout->i_window_height );
+    vout_Control( p_vout, VOUT_SET_SIZE, p_vout->i_window_width,
+                  p_vout->i_window_height );
+
+    /* re-start vout */
+    InitThread( p_vout );
+
+    vlc_mutex_unlock( &p_vout->change_lock );
+
+    /* reset aspect_ratio */
+    var_SetString( p_vout, "aspect-ratio", "" );
+}
+
diff --git a/src/video_output/vout_intf.c b/src/video_output/vout_intf.c
index 89cc527..2a89fff 100644
--- a/src/video_output/vout_intf.c
+++ b/src/video_output/vout_intf.c
@@ -56,7 +56,6 @@
 /*****************************************************************************
  * Local prototypes
  *****************************************************************************/
-static void InitWindowSize( vout_thread_t *, unsigned *, unsigned * );
 
 /* Object variables callbacks */
 static int ZoomCallback( vlc_object_t *, char const *,
@@ -174,7 +173,11 @@ static const struct
 } p_zoom_values[] = {
     { 0.25, N_("1:4 Quarter") },
     { 0.5, N_("1:2 Half") },
+    { 0.75, N_("0.75 Original") },
     { 1, N_("1:1 Original") },
+    { 1.25, N_("1.25 Original") },
+    { 1.5, N_("1.5 Original") },
+    { 1.75, N_("1.75 Original") },
     { 2, N_("2:1 Double") },
     { 0, NULL } };
 
@@ -409,8 +412,8 @@ void vout_IntfInit( vout_thread_t *p_vout )
     p_vout->b_scale = val.b_bool;
 
     /* Initialize the dimensions of the video window */
-    InitWindowSize( p_vout, &p_vout->i_window_width,
-                    &p_vout->i_window_height );
+    vout_InitWindowSize( p_vout, &p_vout->i_window_width,
+                         &p_vout->i_window_height );
 
     /* Add a variable to indicate if the window should be on top of others */
     var_Create( p_vout, "video-on-top", VLC_VAR_BOOL | VLC_VAR_DOINHERIT
@@ -914,13 +917,13 @@ int vout_vaControlDefault( vout_thread_t *p_vout, int i_query, va_list args )
 }
 
 /*****************************************************************************
- * InitWindowSize: find the initial dimensions the video window should have.
+ * vout_InitWindowSize: find the initial dimensions the video window should have.
  *****************************************************************************
  * This function will check the "width", "height" and "zoom" config options and
  * will calculate the size that the video window should have.
  *****************************************************************************/
-static void InitWindowSize( vout_thread_t *p_vout, unsigned *pi_width,
-                            unsigned *pi_height )
+void vout_InitWindowSize( vout_thread_t *p_vout, unsigned *pi_width,
+                                 unsigned *pi_height )
 {
     vlc_value_t val;
     int i_width, i_height;
@@ -993,10 +996,24 @@ static int ZoomCallback( vlc_object_t *p_this, char const *psz_cmd,
 {
     vout_thread_t *p_vout = (vout_thread_t *)p_this;
     (void)psz_cmd; (void)oldval; (void)newval; (void)p_data;
-    InitWindowSize( p_vout, &p_vout->i_window_width,
-                    &p_vout->i_window_height );
-    vout_Control( p_vout, VOUT_SET_SIZE, p_vout->i_window_width,
-                  p_vout->i_window_height );
+
+    vlc_mutex_lock( &p_vout->change_lock );
+
+    if( !p_vout->b_fullscreen )
+    {
+        vout_InitWindowSize( p_vout, &p_vout->i_window_width,
+                             &p_vout->i_window_height );
+
+        vout_Control( p_vout, VOUT_SET_SIZE, p_vout->i_window_width,
+                      p_vout->i_window_height );
+    }
+    else
+    {
+        p_vout->i_changes |= VOUT_ZOOM_CHANGE;
+    }
+
+    vlc_mutex_unlock( &p_vout->change_lock );
+
     return VLC_SUCCESS;
 }
 
@@ -1136,8 +1153,8 @@ static int CropCallback( vlc_object_t *p_this, char const *psz_cmd,
     }
 
  crop_end:
-    InitWindowSize( p_vout, &p_vout->i_window_width,
-                    &p_vout->i_window_height );
+    vout_InitWindowSize( p_vout, &p_vout->i_window_width,
+                         &p_vout->i_window_height );
 
     p_vout->i_changes |= VOUT_CROP_CHANGE;
 
@@ -1217,8 +1234,7 @@ static int ScalingCallback( vlc_object_t *p_this, char const *psz_cmd,
 
     vlc_mutex_lock( &p_vout->change_lock );
 
-    p_vout->b_scale = newval.b_bool;
-    p_vout->i_changes |= VOUT_SIZE_CHANGE;
+    p_vout->i_changes |= VOUT_SCALE_CHANGE;
 
     vlc_mutex_unlock( &p_vout->change_lock );
 
diff --git a/src/video_output/vout_pictures.c b/src/video_output/vout_pictures.c
index ffb2c57..cef72d4 100644
--- a/src/video_output/vout_pictures.c
+++ b/src/video_output/vout_pictures.c
@@ -461,16 +461,26 @@ void vout_PlacePicture( const vout_thread_t *p_vout,
         return;
     }
 
-    if( p_vout->b_scale )
+    bool b_scale = p_vout->b_scale;
+    uint64_t ll_zoom = p_vout->ll_zoom;
+
+    if( b_scale )
     {
         *pi_width = i_width;
         *pi_height = i_height;
     }
-    else
+    else if ( ll_zoom == ZOOM_FP_FACTOR )
     {
         *pi_width = __MIN( i_width, p_vout->fmt_in.i_visible_width );
         *pi_height = __MIN( i_height, p_vout->fmt_in.i_visible_height );
     }
+    else
+    {
+        *pi_width = 
+             p_vout->fmt_in.i_visible_width * ll_zoom / ZOOM_FP_FACTOR;
+        *pi_height =
+             p_vout->fmt_in.i_visible_height * ll_zoom / ZOOM_FP_FACTOR;
+    }
 
      int64_t i_scaled_width = p_vout->fmt_in.i_visible_width * (int64_t)p_vout->fmt_in.i_sar_num *
                               *pi_height / p_vout->fmt_in.i_visible_height / p_vout->fmt_in.i_sar_den;
-- 
1.5.2.5

